# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_alerts_settings_info
short_description: Information module for networks _alerts _settings
description:
  - Get all networks _alerts _settings.
  - Return the alert configuration for this network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
      - NetworkId path parameter. Network ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks getNetworkAlertsSettings
    description: Complete reference of the getNetworkAlertsSettings API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-alerts-settings
notes:
  - SDK Method used are
    networks.Networks.get_network_alerts_settings,
  - Paths used are
    get /networks/{networkId}/alerts/settings,
"""

EXAMPLES = r"""
- name: Get all networks _alerts _settings
  cisco.meraki.networks_alerts_settings_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "alerts": [
        {
          "alertDestinations": {
            "allAdmins": true,
            "emails": [
              "string"
            ],
            "httpServerIds": [
              "string"
            ],
            "smsNumbers": [
              "string"
            ],
            "snmp": true
          },
          "enabled": true,
          "filters": {
            "conditions": [
              {
                "direction": "string",
                "duration": 0,
                "threshold": 0,
                "type": "string",
                "unit": "string"
              }
            ],
            "failureType": "string",
            "lookbackWindow": 0,
            "minDuration": 0,
            "name": "string",
            "period": 0,
            "priority": "string",
            "regex": "string",
            "selector": "string",
            "serials": [
              "string"
            ],
            "ssidNum": 0,
            "tag": "string",
            "threshold": 0,
            "timeout": 0
          },
          "type": "string"
        }
      ],
      "defaultDestinations": {
        "allAdmins": true,
        "emails": [
          "string"
        ],
        "httpServerIds": [
          "string"
        ],
        "snmp": true
      },
      "muting": {
        "byPortSchedules": {
          "enabled": true
        }
      }
    }
"""
